# Vireo [![Build Status](https://travis-ci.org/ni/VireoSDK.svg?branch=incoming)](https://travis-ci.org/ni/VireoSDK)

A compact parallel execution runtime for VIs saved in VI assembly format (.via files).

The Vireo project provides a subset of LabVIEW runtime functionality for smaller targets. An example of its use is the EV3 support in the _LabVIEW Module for LEGO® MINDSTORMS®_. The LabVIEW features supported is primarily defined by the features needed for the VIA files generated by LabVIEW.

# Building

Requirements:
- [Node](https://nodejs.org/en/) (> v4.4.7)
- [Emscripten](https://github.com/kripken/emscripten) (v1.36.0)
- make

Setup:
- `npm install`

#### Visual Studio
Works with Visual Studio 2013 and 2015 (see the `Vireo_VS` folder).

### Easy Build
*`esh & vireo.js`*

From the root directory:
```shell
$ make
```
The `esh` and `vireo.js` files will be put into the `./dist` directory.

#### Build Native Binary
```shell
$ make native
```

#### Build Javascript File
```shell
$ make js
```

# Testing
From the root directory:

`$ make test` - Runs all tests for Native and JS

## Running Tests
From within the `test-it` directory:

#### Against esh target (`-n`)
```shell
$ ./test.js -n
```

#### Against vireo.js target (`-j`)
```shell
$ ./test.js -j
```

#### Running Test Suites (`-t [test suite]`)
```shell
$ ./test.js -n -t <test suite>
```

#### Run Individual Tests (`[Test].via`)
```shell
$ ./test.js HelloWorld.via
```

#### Listing Out Tests (`-l [test suite]`)
Since the test suites can be created recursively from other test suites in the configuration file, the `-l` command line argument will list out all of the tests to be run with the test suite name provided. Example:
```shell
$ ./test.js -l native
```
Will list out all of the tests that would be run against the `native` test suite.

## Test help
```shell
$ ./test.js -h
Usage: ./test.js [options] [via test files]
Options:
 -n                  Run the tests against the native vireo target (esh)
 -j                  Run the tests against the javascript target (vireo.js)
 -t [test suite]     Run the tests in the given test suite
 -l [test suite]     List the tests that would be run in given test suite,
                        or list the test suite options if not provided
 -e                  Execute the test suite or tests provided and show their
                        raw output; do not compute pass/fail
 -h                  Print this usage message
 --once              Will only run the tests once (default is to run twice)
```

## Adding Tests

#### Test Configuration
The `.via` test files are put in the `test-it` folder and the results from the stdout of the test `.via` file from vireo is put in a file of the same name inside the `test-it/results/` folder. The test name is then added to a test suite within the `testList.json` file in the `test-it` directory.

The `testList.json` file has two properties that are required for each test suite name:

##### include
This is an array of strings that are names to other test suites. These test suite names are processed recursively to add the other tests together into one list of tests to run. (Duplicates are omitted if overlaps exists between tests)

##### tests
This is an array of strings that contain the list of `.via` files that the test suite should run.

#### Adding a Test Suite
This will be a simple example that will add the test suite `rpi` with the `RpiTest.via` file to the test manager.

1. Put the `RpiTest.via` in the `test-it/` folder and put the `RpiTest.vtr` file in the `test-it/results/` folder.
2. Then to add the `rpi` test suite to the testing manager, add this example code to the `testList.json` file:
```json
"rpi": {
    "include": [ "common" ],
    "tests": [
        "RpiTest.via"
    ]
}
```
This will add a test suite `rpi` that will include the test `RpiTest.via` and all of the tests included in the `common` test suite.
3. Try it out to verify it all works and your tests pass:
```shell
$ ./test.js -n -t rpi
```

# Legal
Features beyond that core set, that can be accessed directly from VIA source written by hand, should be considered experimental, and subject to change at any time. A complete list of disclaimers and terms is described in LICENSE.txt

